/* erpcd-talker: minimal routines to talk to an erpcd via UDP.
 *
 * Copyright (c) David Luyer, 1999
 */

#define INTERACTIVE 1

#include <stdio.h>
#include <netdb.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <string.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <sys/socket.h>

static void die(int sig) {
  fprintf(stderr, "Request failed.\n");
  exit(-1);
}

int main(int argc, char ** argv) {
  int sd;
#if 0
  int id, seq;
#endif
  struct sockaddr_in Server;
  char buf1[64] = { 0x01, 0x02, 0x03, 0x04,
                    0x00, 0x20, 0x00, 0x00,
                    0x00, 0x00, 0x00, 0x00,
                    0x00, 0x03, 0x00, 0x01,
		    0x00, 0x0f, /* srpc init */
		    0x00, 0x00, 0x00, 0x00, /* srpc_id */
		    0x00, 0x00, 0x00, 0x00, /* sequence # */
		    0x00 }; /* truncated key to be ignored */
  char buf2[187] = { 0x01, 0x02, 0x03, 0x04,
                     0x00, 0x20, 0x00, 0x00,
                     0x00, 0x00, 0x00, 0x00,
                     0x00, 0x03, 0x00, 0x01,
		     0x00, 0x15, /* req dialup addr */
		     0x00, 0x00, 0x00, 0x00, /* srpc_id */
		     0x00, 0x00, 0x00, 0x00, /* sequence # */
		     0x00, 0x00, 0x00, 0x00, /* handle */
		     0x00, 0x00, 0x00, 0x00, /* logid */
		     0x00, 0x00, 0x00, 0x00, /* inet */
		     0x00, 0x00, /* port */
		     0x00, 0x06, /* service */
		     0x00, 0x00, 0x00, 0x00, /* loc */
		     0x00, 0x00, 0x00, 0x00, /* rem */
		     /* + 129 user */
		     /* + 6 node */
		     /* + 2 alignment (dummy) */ };

  /* BUG: udp packets aren't always accepted by the same process
   *      so sometimes they id is 0.  so use 0. */
#if 0
  srandom(getpid() ^ time(NULL));
  *(int *)(buf1+18) = id = random();
  *(int *)(buf1+22) = seq = random();
  seq = htonl((htonl(seq) + 1));
  *(int *)(buf2+18) = id;
  *(int *)(buf2+22) = seq;
#endif
  memset(&Server, 0, sizeof(Server));
  Server.sin_family = AF_INET;
  Server.sin_port = htons(121);
  if (argc != 2) {
    fprintf(stderr, "Usage: %s hostname\n", argv[0]);
    return 1;
  } else {
    if ((Server.sin_addr.s_addr = inet_addr(argv[1])) == -1) {
      struct hostent *host;
      if ((host = gethostbyname(argv[1])) == NULL) {
	fprintf(stderr, "Failed server lookup: %s\n", argv[1]);
        return 1;
      }
      memcpy(&Server.sin_addr.s_addr, host->h_addr_list[0], 4);
    }
  }
#if INTERACTIVE
  printf("username: "); fflush(stdout);
#endif
  buf2[read(0, buf2+50, 32) + 50 - 1] = '\0';
  if (!*(buf2+50)) {
    fprintf(stderr, "Username cannot be blank.\n");
    return 1;
  }
  if ((sd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
    perror("socket");
    return 1;
  }
  signal(SIGALRM, die);
  alarm(60);
  if (sendto(sd, buf1, 27, 0, &Server, sizeof(Server)) < 0) {
    perror("sendto");
    return 1;
  }
  if (recv(sd, buf1, sizeof(buf1), 0) < 0) {
    perror("recv");
    return 1;
  }
  if(*(buf1+4+2+1) != 2) {
    fprintf(stderr, "Problem opening session with server.\n");
    return 1;
  }
  if (sendto(sd, buf2, 187, 0, &Server, sizeof(Server)) < 0) {
    perror("sendto");
    return 1;
  }
  if (recv(sd, buf2, sizeof(buf2), 0) != 42) {
    perror("recv");
    return 1;
  }
  if(*(buf2+4+2+1) != 2 || *(int *)(buf2+22) != htonl(0xbabe3333ul)) {
    fprintf(stderr, "Problem getting IP response from server.\n");
    return 1;
  }
  if (recv(sd, buf2, sizeof(buf2), 0) != 50) {
    perror("recv");
    return 1;
  }
  if(*(buf2+4+2+1) != 0 || (*(int *)(buf2+30) != htonl(0xface1111ul) &&
                            *(int *)(buf2+30) != htonl(0xdeaf2222ul))) {
    fprintf(stderr, "Problem getting IP address from server.\n");
    return 1;
  }
  if(*(int *)(buf2+30) != htonl(0xface1111ul)) {
    fprintf(stderr, "No IP allocated.\n");
    return 1;
  }
#if INTERACTIVE
  printf("IP Address: %s.\n", inet_ntoa(*(struct in_addr *)(buf2+38)));
#else
  printf("%s\n", inet_ntoa(*(struct in_addr *)(buf2+38)));
#endif
  close(sd);
  return 0;
}
